//===========================================================================
//
//  Code written for PSoC6 on a Pioneer BLE board
//
//  CRYSTALFONTZ CFAF240240A1-013T 240X240 SPI COLOR 1.3" TFT
//  ref: https://www.crystalfontz.com/product/cfaf240240a1013t
//
//  This code uses 4-wire SPI mode.
//
// The controller is a Sitronix ST7789H2
//   http://www.crystalfontz.com/controllers/Sitronix/ST7789H2
//
//  2018-10-19 Brent A. Crosby
//===========================================================================
//This is free and unencumbered software released into the public domain.
//
//Anyone is free to copy, modify, publish, use, compile, sell, or
//distribute this software, either in source code form or as a compiled
//binary, for any purpose, commercial or non-commercial, and by any
//means.
//
//In jurisdictions that recognize copyright laws, the author or authors
//of this software dedicate any and all copyright interest in the
//software to the public domain. We make this dedication for the benefit
//of the public at large and to the detriment of our heirs and
//successors. We intend this dedication to be an overt act of
//relinquishment in perpetuity of all present and future rights to this
//software under copyright law.
//
//THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
//EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
//MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
//IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR
//OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
//ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
//OTHER DEALINGS IN THE SOFTWARE.
//
//For more information, please refer to <http://unlicense.org/>
//============================================================================
//
// Display is Crystalfontz  CFAF240240A1-013T 240X240 SPI COLOR 1.3" TFT
//   https://www.crystalfontz.com/product/cfaf240240a1013t
//
//============================================================================
#include "project.h"
#include "CFAF240240A_LCD.h"

// Function |LCD | PSoC  | Wire
// LCD_MOSI | 10 | P10.0 | Green
// LCD_MISO | 11 | P10.1 | Blue
// LCD_SCK  |  9 | P10.2 | White
// LCD_nCS  | 13 | P10.3 | Grey
// LCD_DnC  | 12 | P10.4 | Yellow
// LCD_nRST | 15 | P10.5 | Purple

//This is the 16-bit R5-G6-B5 frame buffer in the PSoC's RAM
uint16_t framebuffer[240][240];

//============================================================================
//Port control macros
#define SET_MOSI Cy_GPIO_Write(LCD_MOSI_PORT,LCD_MOSI_NUM,1);
#define CLR_MOSI Cy_GPIO_Write(LCD_MOSI_PORT,LCD_MOSI_NUM,0);
#define SET_SCK  Cy_GPIO_Write(LCD_SCK_PORT, LCD_SCK_NUM, 1);
#define CLR_SCK  Cy_GPIO_Write(LCD_SCK_PORT, LCD_SCK_NUM, 0);
#define SET_nCS  Cy_GPIO_Write(LCD_nCS_PORT, LCD_nCS_NUM, 1);
#define CLR_nCS  Cy_GPIO_Write(LCD_nCS_PORT, LCD_nCS_NUM, 0);
#define SET_DnC  Cy_GPIO_Write(LCD_DnC_PORT, LCD_DnC_NUM, 1);
#define CLR_DnC  Cy_GPIO_Write(LCD_DnC_PORT, LCD_DnC_NUM, 0);
#define SET_nRST Cy_GPIO_Write(LCD_nRST_PORT,LCD_nRST_NUM,1);
#define CLR_nRST Cy_GPIO_Write(LCD_nRST_PORT,LCD_nRST_NUM,0);
//============================================================================
void SPI_sendCommand(uint8_t command)
  {
  // Select the LCD's command register
  CLR_DnC;
  // Select the LCD controller
  CLR_nCS;
  //Send the command via SPI:
  LCD_SPI_Write(command);
  //Wait for the FIFO and TX Shift regieter to be empty before changing
  //the state of the control lines.
  while(0 == LCD_SPI_IsTxComplete());
  // Deselect the LCD controller
  SET_nCS;
  }
//----------------------------------------------------------------------------
void SPI_sendData(uint8_t data)
  {
  // Select the LCD's data register
  SET_DnC;
  // Select the LCD controller
  CLR_nCS;
  //Send the data via SPI:
  LCD_SPI_Write(data);
  //Wait for the FIFO and TX Shift regieter to be empty before changing
  //the state of the control lines.
  while(0 == LCD_SPI_IsTxComplete());
  // Deselect the LCD controller
  SET_nCS; 
  }
//============================================================================
// Defines for the ST7789 registers.
// ref: http://www.crystalfontz.com/controllers/Sitronix/ST7789H2
#define ST7789H2_10_SLPIN      (0x10)
#define ST7789H2_11_SLPOUT     (0x11)
#define ST7789H2_20_INVOFF     (0x20)
#define ST7789H2_21_INVON      (0x21)
#define ST7789H2_28_DISPOFF    (0x28)
#define ST7789H2_29_DISPON     (0x29)
#define ST7789H2_2A_CASET      (0x2A)
#define ST7789H2_2B_RASET      (0x2B)
#define ST7789H2_2C_RAMWR      (0x2C)
#define ST7789H2_36_MADCTL     (0x36)
#define ST7789H2_3A_COLMOD     (0x3A)
#define ST7789H2_B2_PORCTRL    (0xB2)
#define ST7789H2_B7_GCTRL      (0xB7)
#define ST7789H2_BB_VCOMS      (0xBB)
#define ST7789H2_C0_LCMCTRL    (0xC0)  //LCM Control
#define ST7789H2_C2_VDVVRHEN   (0xC2)  //VDV and VRH Command Enable
#define ST7789H2_C3_VRHS       (0xC3)  //VRH Set
#define ST7789H2_C4_VDVSET     (0xC4)  //VDV Set
#define ST7789H2_C6_FRCTR2     (0xC6)  //Frame Rate Control in Normal Mode
#define ST7789H2_D0_PWCTRL1    (0xD0)  //Power Control 1
#define ST7789H2_E0_PVGAMCTRL  (0xE0)
#define ST7789H2_E1_NVGAMCTRL  (0xE1)
//============================================================================
void LCD_Initialize(void)
  {
  //Reset the LCD controller
  CLR_nRST;
  CyDelay(1);//10µS min
  SET_nRST;
  CyDelay(120);//120mS max
  
  //SLPOUT (11h): Sleep Out ("Sleep Out"  is chingrish for "wake")
  //The DC/DC converter is enabled, Internal display oscillator
  //is started, and panel scanning is started.
  SPI_sendCommand(ST7789H2_11_SLPOUT);
  CyDelay(120);

  //-----------------------------Display setting--------------------------------
  //Address control
  SPI_sendCommand(ST7789H2_36_MADCTL); //Page 215
  //SPI_sendData(0x00); //DEFAULT
  //SPI_sendData(0x48); //TEST
  SPI_sendData(0x00);
  
  // Bit D7- Page Address Order
  // “0” = Top to Bottom (When MADCTL D7=”0”).
  // “1” = Bottom to Top (When MADCTL D7=”1”).
  // Bit D6- Column Address Order
  // “0” = Left to Right (When MADCTL D6=”0”).
  // “1” = Right to Left (When MADCTL D6=”1”).
  // Bit D5- Page/Column Order
  // “0” = Normal Mode (When MADCTL D5=”0”).
  // “1” = Reverse Mode (When MADCTL D5=”1”)
  // Note: Bits D7 to D5, alse refer to section 8.12 Address Control
  // Bit D4- Line Address Order
  // “0” = LCD Refresh Top to Bottom (When MADCTL D4=”0”)
  // “1” = LCD Refresh Bottom to Top (When MADCTL D4=”1”)
  // Bit D3- RGB/BGR Order
  // “0” = RGB (When MADCTL D3=”0”)
  // “1” = BGR (When MADCTL D3=”1”)
  // Bit D2- Display Data Latch Data Order
  // “0” = LCD Refresh Left to Right (When MADCTL D2=”0”)
  // “1” = LCD Refresh Right to Left (When MADCTL D2=”1”)  
  
  //Interface pixel format Pg 224
  SPI_sendCommand(ST7789H2_3A_COLMOD);
  SPI_sendData(0x05);
  // 0ddd 0iii
  // |||| ||||--iii interface color
  // |||| |     011 = 12-bit/pixel
  // |||| |     101 = 16-bit/pixel * 
  // |||| |     110 = 18-bit/pixel
  // |||| |     111 = 16M truncated
  // |||| |-----0
  // ||||-------ddd RGB color
  // |          101 = 65K  RGB interface
  // |          110 = 262K  RGB interface
  // |----------0
  
  SPI_sendCommand(ST7789H2_21_INVON);

  //------------------------- Frame rate setting-------------------
  // Porch Control
  SPI_sendCommand(ST7789H2_B2_PORCTRL);	
	SPI_sendData(0x0C);
	SPI_sendData(0x0C);
	SPI_sendData(0x00);
	SPI_sendData(0x33);
	SPI_sendData(0x33);

	//Gate Control
  SPI_sendCommand(ST7789H2_B7_GCTRL);	
	SPI_sendData(0x35);							//VGH:13.26/VGL:-10.43

  //----------------------- Power setting-------------------
  //VCOM Setting
  SPI_sendCommand(ST7789H2_BB_VCOMS);
  SPI_sendData(0x1F);     //VCOM = 0.875V

  //LCM Control
  SPI_sendCommand(ST7789H2_C0_LCMCTRL);
  SPI_sendData(0x2C);     //Power On Sequence default

	//VDV and VRH Command Enable
  SPI_sendCommand(ST7789H2_C2_VDVVRHEN);
  SPI_sendData(0x01);

	//VRH Set
  SPI_sendCommand(ST7789H2_C3_VRHS);
  SPI_sendData(0x12);     //4.45+( vcom+vcom offset+0.5vdv)

	//VDV Set
  SPI_sendCommand(ST7789H2_C4_VDVSET);  
  SPI_sendData(0x20);     //VDV = 0V

  //Frame Rate Control in Normal Mode
  SPI_sendCommand(ST7789H2_C6_FRCTR2);  
  SPI_sendData(0x0F);     //60HZ

	SPI_sendCommand(ST7789H2_D0_PWCTRL1);   //Power Control 1
  SPI_sendData(0xA4);
  SPI_sendData(0xA1);     //VDS=2.3V/AVCL = -4.8V /AVDD=6.8V

  //Gamma tables
	SPI_sendCommand(ST7789H2_E0_PVGAMCTRL);
  SPI_sendData(0xD0);
  SPI_sendData(0x08);
  SPI_sendData(0x11);
  SPI_sendData(0x08);
  SPI_sendData(0x0C);
  SPI_sendData(0x15);
  SPI_sendData(0x39);
  SPI_sendData(0x33);
  SPI_sendData(0x50);
  SPI_sendData(0x36);
  SPI_sendData(0x13);
  SPI_sendData(0x14);
  SPI_sendData(0x29);
  SPI_sendData(0x2D);
  
  //--------------------
	SPI_sendCommand(ST7789H2_E1_NVGAMCTRL);
  SPI_sendData(0xD0);
  SPI_sendData(0x08);
  SPI_sendData(0x10);
  SPI_sendData(0x08);
  SPI_sendData(0x06);
  SPI_sendData(0x06);
  SPI_sendData(0x39);
  SPI_sendData(0x44);
  SPI_sendData(0x51);
  SPI_sendData(0x0B);
  SPI_sendData(0x16);
  SPI_sendData(0x14);
  SPI_sendData(0x2F);
  SPI_sendData(0x31);

  // Exit Sleep Mode
	SPI_sendCommand(ST7789H2_11_SLPOUT); 
	CyDelay(120);
  // Display on
	SPI_sendCommand(ST7789H2_29_DISPON); 
  }
//============================================================================
// This function resets the display "cursor position" using non-DMA SPI
// transfers, and prepares the controller to receive the main display data
// over DMA.
void LCD_DMA_Frame_Start(void)
  {
  //CASET (2Ah): Column Address Set
  // * The value of XS [15:0] and XE [15:0] are referred when RAMWR
  //   command comes.
  // * Each value represents one column line in the Frame Memory.
  // * XS [15:0] always must be equal to or less than XE [15:0]
  SPI_sendCommand(ST7789H2_2A_CASET);	// Column Address Set
  //Write the parameters for the "column address set" command
  SPI_sendData(0);   // X address start, 0:0
  SPI_sendData(0);   //
  SPI_sendData(0);   // X address end: 0:239
  SPI_sendData(239); //

  //Write the "row address set" command to the LCD
  //RASET (2Bh): Row Address Set
  // * The value of YS [15:0] and YE [15:0] are referred when RAMWR
  //   command comes.
  // * Each value represents one row line in the Frame Memory.
  // * YS [15:0] always must be equal to or less than YE [15:0]
  SPI_sendCommand(ST7789H2_2B_RASET); //Row address set
  //Write the parameters for the "row address set" command
  SPI_sendData(0);   // Y address start, 0:0
  SPI_sendData(0);   //
  SPI_sendData(0);   // Y address end: 0:239
  SPI_sendData(239); //

  // //Write the "write data" command to the LCD
  // //RAMWR (2Ch): Memory Write
  SPI_sendCommand(ST7789H2_2C_RAMWR); //write data
  
  // Select the LCD controller's data register, in preparation for
  // the DMA transfer.
  SET_DnC;
  CLR_nCS;  
  }
//============================================================================
void LCD_DMA_Frame_End(void)
  {
  // De-select the LCD controller.
  SET_nCS;  
  }
//============================================================================
uint16_t LCD_R5G6B5_from_RGB(uint8_t r,uint8_t g,uint8_t b)
  {
  return((((uint16_t)(r & 0xF8))<<8) |
         (((uint16_t)(g & 0xFC))<<3) |
         (((uint16_t)(b & 0xF8))>>3));
  }
//============================================================================
//
//  These are DEMO routines. Not really necessary.
//
//============================================================================
void LCD_FillFrameBuffer(uint16_t color)
  {
  //The byte order of words in the ARM RAM is different than the byte
  //order of words in the LCD RAM (endian).
  color=((color&0xFF00) >> 8 ) | ((color&0x00FF) << 8 );    
    
  uint8_t
    x;
  uint8_t
    y;
	for(y=0;y<240;y++)
	  {
		for(x=0;x<240;x++)
		  {
   		framebuffer[y][x]=color;
      }
	  }
  }
//============================================================================
void LCD_Flash_Image_to_Framebuffer(const uint16_t image[240][240])
  {
  //Transfer the data from flash to ram, using 32-bit transfers.
  uint32_t
    i;
  uint32_t
    *dst;
  uint32_t
    *src;
    
  src=(uint32_t *)image;
  dst=(uint32_t *)framebuffer;
    
  // 28800 = 240*240*2 / 4
  for(i=0;i<28800;i++)
    {
    *dst=*src;
    dst++;
    src++;
    }
  //Alternatively, there is probably some memcpy() analog, or you could
  //even use DMA memory-to-memory.
  }  
//============================================================================
// From: http://en.wikipedia.org/wiki/Midpoint_circle_algorithm
void LCD_Circle(uint8_t x0, uint8_t y0, uint8_t radius, uint16_t color)
  {
  uint8_t x = radius;
  uint8_t y = 0;
  int16_t radiusError = 1 - (int16_t) x;

  //The byte order of words in the ARM RAM is different than the byte
  //order of words in the LCD RAM (endian).
  color=((color&0xFF00) >> 8 ) | ((color&0x00FF) << 8 );    
  
  while (x >= y)
    {
    //11 O'Clock
    framebuffer[y0 + x][x0 - y]=color;
    //1 O'Clock
    framebuffer[y0 + x][x0 + y]=color;
    //10 O'Clock
    framebuffer[y0 + y][x0 - x]=color;
    //2 O'Clock
    framebuffer[y0 + y][x0 + x]=color;
    //8 O'Clock
    framebuffer[y0 - y][x0 - x]=color;
    //4 O'Clock
    framebuffer[y0 - y][x0 + x]=color;
    //7 O'Clock
    framebuffer[y0 - x][x0 - y]=color;
    //5 O'Clock
    framebuffer[y0 - x][x0 + y]=color;
    y++;
    if (radiusError < 0)
      radiusError += (int16_t)(2 * y + 1);
    else
      {
      x--;
      radiusError += 2 * (((int16_t) y - (int16_t) x) + 1);
      }
    }
  }
//============================================================================
void LCD_Circle_Demo(void)
  {
  uint8_t
    i;
  //blue background
  LCD_FillFrameBuffer(LCD_R5G6B5_from_RGB(0,0,255));

  //Sweep red to green, through yellow.
  uint8_t
    r;
  uint8_t
    g;
  for(i=6;i<=114;i+=6)
    {
    r=240-(i<<1);
    g=i<<1;
    LCD_Circle(120,120,i,LCD_R5G6B5_from_RGB(r,g,0));
    }
  }
//============================================================================
// From: http://rosettacode.org/wiki/Bitmap/Bresenham's_line_algorithm#C
void LCD_Line(uint16_t x0, uint16_t y0,
              uint16_t x1, uint16_t y1,
              uint16_t color)
  {
  int16_t dx;
  int16_t sx;
  int16_t dy;
  int16_t sy;
  int16_t err;
  int16_t e2;

  //The byte order of words in the ARM RAM is different than the byte
  //order of words in the LCD RAM (endian).
  color=((color&0xFF00) >> 8 ) | ((color&0x00FF) << 8 );    
    
  dx = (int16_t )x1 - (int16_t )x0;
  if(dx < 0)
    dx = -dx;
  sx = x0 < x1 ? 1 : -1;
  dy = (int16_t )y1 - (int16_t )y0;
  if(dy < 0)
    dy = -dy;
  sy = y0 < y1 ? 1 : -1;
  err = (dx > dy ? dx : -dy) / 2;

  for (;;)
    {
    framebuffer[y0][x0]=color;
    if ((x0 == x1) && (y0 == y1))
      {
      break;
      }
    e2 = err;
    if (e2 > -dx)
      {
      err -= dy;
      x0 = (uint16_t)((int16_t) x0 + sx);
      }
    if (e2 < dy)
      {
      err += dx;
      y0 = (uint16_t)((int16_t) y0 + sy);
      }
    }
  }
//============================================================================
void LCD_Line_Demo(void)
  {
#define MAX_X (239)
#define MAX_Y (239)
#define CTR_X (120)
#define CTR_Y (120)
  uint8_t
    x;
  uint8_t
    y;
  uint8_t
    r;
  uint8_t
    g;
  uint8_t
    b;
  uint8_t
    i;

  r = 0xff;
  g = 0x00;
  b = 0x80;
  for (i = 0; i < 10; i++)
    {
    for (x = 0; x <= MAX_X; x++)
      {
      LCD_Line(CTR_X, CTR_Y,
               x, 0,
               LCD_R5G6B5_from_RGB(r++, g--, b += 2));
      }
    for (y = 0; y <= MAX_Y; y++)
      {
      LCD_Line(CTR_X, CTR_Y,
               MAX_X, y,
               LCD_R5G6B5_from_RGB(r++, g += 4, b += 2));
      }
    for (x = MAX_X; 0 != x; x--)
      {
      LCD_Line(CTR_X, CTR_Y,
               x, MAX_Y,
               LCD_R5G6B5_from_RGB(r -= 3, g -= 2, b -= 1));
      }
    for (y = MAX_Y; 0 != y; y--)
      {
      LCD_Line(CTR_X, CTR_Y,
               0, y,
               LCD_R5G6B5_from_RGB(r + -3, g--, b++));
      }
    }
  }
//============================================================================
  