//===========================================================================
//
//  Code written for PSoC6 on a Pioneer BLE board
//
//  CRYSTALFONTZ CFAF240240A1-013T 240X240 SPI COLOR 1.3" TFT
//  ref: https://www.crystalfontz.com/product/cfaf240240a1013t
//
//  This code uses 4-wire SPI mode.
//
// The controller is a Sitronix ST7789H2
//   http://www.crystalfontz.com/controllers/Sitronix/ST7789H2
//
//  2018-10-19 Brent A. Crosby
//===========================================================================
//This is free and unencumbered software released into the public domain.
//
//Anyone is free to copy, modify, publish, use, compile, sell, or
//distribute this software, either in source code form or as a compiled
//binary, for any purpose, commercial or non-commercial, and by any
//means.
//
//In jurisdictions that recognize copyright laws, the author or authors
//of this software dedicate any and all copyright interest in the
//software to the public domain. We make this dedication for the benefit
//of the public at large and to the detriment of our heirs and
//successors. We intend this dedication to be an overt act of
//relinquishment in perpetuity of all present and future rights to this
//software under copyright law.
//
//THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
//EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
//MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
//IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR
//OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
//ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
//OTHER DEALINGS IN THE SOFTWARE.
//
//For more information, please refer to <http://unlicense.org/>
//============================================================================
//
// Display is Crystalfontz  CFAF240240A1-013T 240X240 SPI COLOR 1.3" TFT
//   https://www.crystalfontz.com/product/cfaf240240a1013t
//
//============================================================================
#include "project.h"
#include "CFAF240240A_LCD.h"

//The DMA cannot get it all done in one go, so we have to break it up
//into two sections.
//Keep track of what phase of the transfer we are in
volatile uint8_t SPI_DMA_LCD_status;
#define SPI_DMA_LCD_IDLE        (0)  //public
#define SPI_DMA_LCD_TOP_HALF    (1)  //private
#define SPI_DMA_LCD_BOTTOM_HALF (2)  //private
//============================================================================
#define RED_LED_ON    Cy_GPIO_Write(RedLED_PORT,  RedLED_NUM, 0)
#define RED_LED_OFF   Cy_GPIO_Write(RedLED_PORT,  RedLED_NUM, 1)
static void FailToHandleError(void)
  {   
  // Disable all interrupts.
  __disable_irq();
    
  //Blink DevBoard red LED -- frantically
  while(1)
    {
    RED_LED_ON;
    CyDelay(100);
    RED_LED_OFF;      
    CyDelay(100);
    }
  }
//============================================================================
// This ISR gets hit when the DMA completes.  
static void TxDmaComplete(void)
  {
  // Check tx DMA status
  if((CY_DMA_INTR_CAUSE_COMPLETION !=
      Cy_DMA_Channel_GetStatus(txDMA_HW, txDMA_DW_CHANNEL)) &&
     (CY_DMA_INTR_CAUSE_CURR_PTR_NULL !=
      Cy_DMA_Channel_GetStatus(txDMA_HW, txDMA_DW_CHANNEL)))
    {
    //DMA error occured while TX operations
    FailToHandleError();
    }
  
  // Clear tx DMA interrupt
  Cy_DMA_Channel_ClearInterrupt(txDMA_HW, txDMA_DW_CHANNEL);   
  
  //If we just finished the first half, switch to the second.
  if(SPI_DMA_LCD_TOP_HALF == SPI_DMA_LCD_status)
    {
    //Remember that we are transferring the second half
    SPI_DMA_LCD_status = SPI_DMA_LCD_BOTTOM_HALF;     

    Cy_DMA_Descriptor_SetSrcAddress(
      &txDMA_txDMA_Descriptor,
      (uint8_t *)&(framebuffer[120][0]));
    
    // Enable tx DMA channel.
    Cy_DMA_Channel_Enable(txDMA_HW, txDMA_DW_CHANNEL);
    }
  else
    {
    //If we just finished the second half, idle the LCD and stop.
    if(SPI_DMA_LCD_BOTTOM_HALF == SPI_DMA_LCD_status)
      {
      //Let go of the LCD -- complete the SPI transaction.
      LCD_DMA_Frame_End();
      //Remember that we are done
      SPI_DMA_LCD_status = SPI_DMA_LCD_IDLE;     
      }
    }
  }
//============================================================================
void Initialize_SPI_DMA_for_LCD(void)
  {
  cy_en_scb_spi_status_t
    initStatus;
  initStatus = Cy_SCB_SPI_Init(LCD_SPI_HW, &LCD_SPI_config, &LCD_SPI_context);
  if(initStatus != CY_SCB_SPI_SUCCESS)
    {
    FailToHandleError();
    }
  // Set active slave select to line 0
  Cy_SCB_SPI_SetActiveSlaveSelect(LCD_SPI_HW, LCD_SPI_SPI_SLAVE_SELECT0);
  }
//============================================================================
void Start_SPI_DMA_transfer_to_LCD(void)
  {
  //Remember that we are transferring
  SPI_DMA_LCD_status = SPI_DMA_LCD_TOP_HALF;     
    
  // Initialize descriptor
  cy_en_dma_status_t
    dma_init_status;
  dma_init_status=
    Cy_DMA_Descriptor_Init(&txDMA_txDMA_Descriptor,
                           &txDMA_txDMA_Descriptor_config);
  if(dma_init_status!=CY_DMA_SUCCESS)
    {
    FailToHandleError();
    }

  // Initialize the DMA channel
	cy_stc_dma_channel_config_t
    channelConfig;	
  channelConfig.descriptor  = &txDMA_txDMA_Descriptor;
  channelConfig.preemptable = txDMA_PREEMPTABLE;
  channelConfig.priority    = txDMA_PRIORITY;
  channelConfig.enable      = false;
    
  dma_init_status = 
    Cy_DMA_Channel_Init(txDMA_HW, txDMA_DW_CHANNEL, &channelConfig);
  if(dma_init_status!=CY_DMA_SUCCESS)
    {
    FailToHandleError();
    }
    
  // Set source and destination for descriptor 1
  Cy_DMA_Descriptor_SetSrcAddress(
    &txDMA_txDMA_Descriptor,
    (uint8_t *)&(framebuffer[0][0]));
  Cy_DMA_Descriptor_SetDstAddress(
    &txDMA_txDMA_Descriptor,
    (void *)&LCD_SPI_HW->TX_FIFO_WR);
    
  // Initialize and enable the interrupt from TxDma
  Cy_SysInt_Init(&intTxDMA_cfg, &TxDmaComplete);
  NVIC_EnableIRQ(intTxDMA_cfg.intrSrc);
      
  //Enable DMA interrupt source.
  Cy_DMA_Channel_SetInterruptMask(
    txDMA_HW,
    txDMA_DW_CHANNEL,
    txDMA_INTR_MASK);
  Cy_DMA_Enable(txDMA_HW);    
  
  //Get the LCD ready for the barrage of data
  LCD_DMA_Frame_Start();
        
  // Enable tx DMA channel.
  Cy_DMA_Channel_Enable(txDMA_HW, txDMA_DW_CHANNEL);    
  }
//============================================================================
  